Exception: Minisky::ExpiredTokenError

Inherits:
ClientErrorResponse show all
Defined in:
lib/minisky/errors.rb

Overview

Raised when the API returns an error indicating that the access or request token that was passed in the header is expired.

Instance Attribute Summary

Attributes inherited from BadResponse

#data, #status

Method Summary

Methods inherited from BadResponse

#error_message, #error_type, #initialize

Constructor Details

This class inherits a constructor from Minisky::BadResponse