Exception: Minisky::ExpiredTokenError
- Inherits:
-
ClientErrorResponse
- Object
- StandardError
- Error
- BadResponse
- ClientErrorResponse
- Minisky::ExpiredTokenError
- Defined in:
- lib/minisky/errors.rb
Overview
Raised when the API returns an error indicating that the access or request token that was passed in the header is expired.
Instance Attribute Summary
Attributes inherited from BadResponse
Method Summary
Methods inherited from BadResponse
#error_message, #error_type, #initialize
Constructor Details
This class inherits a constructor from Minisky::BadResponse